@@ -2,38 +2,43 @@ package Class::Superclasses;
 
 use strict;
 use warnings;
+
 use List::Util qw(first);
 use PPI;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 sub new{
     my ($class,$doc) = @_,
     my $self         = {};
+
     bless $self,$class;
     
     $self->document($doc);
     
     return $self;
-}# new
+}
 
 sub superclasses{
     my ($self) = @_; 
     return wantarray ? @{$self->{super}} : $self->{super};
-}# superclasses
+}
 
 sub document{
     my ($self,$doc) = @_;
+
     if(defined $doc){
         $self->{document} = $doc;
         $self->{super}    = $self->_find_super($doc);
     }
-}# document
+
+    return $self;
+}
 
 sub _find_super{
     my ($self,$doc) = @_;
+
     my $ppi    = PPI::Document->new($doc) or die $!;
-    
     my $varref = $ppi->find('PPI::Statement::Variable');
     my @vars   = ();
 
@@ -41,75 +46,78 @@ sub _find_super{
         @vars    = $self->_get_isa_values($varref);
     }
     
-    my $baseref  = $ppi->find('PPI::Statement::Include');
-    my @base     = ();
+    my $baseref  = $ppi->find('PPI::Statement::Include') || [];
     my @includes = qw(base parent);
-
-    if($baseref){
-        @base = $self->_get_include_values([grep{my $i = $_->module; grep{ $_ eq $i }@includes }@$baseref]);
-    }
+    my @base     = $self->_get_include_values([grep{my $i = $_->module; grep{ $_ eq $i }@includes }@$baseref]);
 
     my @moose;
-    if ( $baseref && first{ $_->module eq 'Moose' or $_->module eq 'Moo' }@$baseref ) {
-        my @extends = grep{ $_->schild(0)->content eq "extends" }@{ $ppi->find('PPI::Statement') || [] };
+    my @moose_like_modules = qw(Moose Moo Mouse Mo);
+    my $is_moose;
 
-        for my $extend ( @extends ) {
-            push @moose, $self->_get_moose_values( $extend );
+    for my $base_class ( @{$baseref} ) {
+        if ( first{ $base_class->module eq $_ }@moose_like_modules ) {
+
+            for my $stmt ( @{ $ppi->find('PPI::Statement') } ) {
+                push @moose, $self->_get_moose_values( $stmt );
+            }
         }
     }
 
     return [@vars, @base, @moose];
-} # _find_super
+}
 
 sub _get_moose_values{
     my ($self,$elem) = @_;
 
     my @parents;
 
+    return if $elem->schild(0)->content ne 'extends';
+
     if ( $elem->find_any('PPI::Statement::Expression') ) {
         push @parents, $self->_parse_expression( $elem );
     }
     elsif ( $elem->find_any('PPI::Token::QuoteLike::Words') ) {
         push @parents, $self->_parse_quotelike( $elem );
     }
-    elsif ( $elem->find_any( 'PPI::Structure::List' ) ) {
-        push @parents, $self->_parse_list( $elem );
-    }
     elsif( $elem->find( \&_any_quotes ) ){
         push @parents, $self->_parse_quotes( $elem );
     }
 
     return @parents;
-}# _get_values
+}
 
 sub _get_include_values{
-    my ($self,$baseref) = @_;
+    my ($self, $baseref) = @_;
     my @parents;
 
-    for my $base(@$baseref){
+    BASE:
+    for my $base( @{$baseref} ){
         my @tmp_array;
 
         if( $base->find_any('PPI::Statement::Expression') ){
-            push(@parents,$self->_parse_expression( $base ));
+            push @tmp_array, $self->_parse_expression( $base );
         }
         elsif( $base->find_any('PPI::Token::QuoteLike::Words') ){
-            push(@parents,$self->_parse_quotelike( $base ));
+            push @tmp_array, $self->_parse_quotelike( $base );
         }
         elsif( $base->find( \&_any_quotes ) ){
-            push @parents,$self->_parse_quotes( $base );
+            push @tmp_array, $self->_parse_quotes( $base );
+        }
+
+        if ( $base->module eq 'parent' ) {
+            @tmp_array = grep{ $_ ne '-norequire' }@tmp_array;
         }
 
-        @tmp_array = grep{ $_ ne '-norequire' }@tmp_array if $base->module eq 'parent';
         push @parents, @tmp_array;
     }
 
     return @parents;
-}# _get_base_values
+}
 
 sub _any_quotes{
     my ($parent,$elem) = @_;
-    
-    $parent == $elem->parent and (
+
+    $parent eq $elem->parent and (
         $elem->isa( 'PPI::Token::Quote::Double' ) or
         $elem->isa( 'PPI::Token::Quote::Single' )
     );
@@ -118,44 +126,39 @@ sub _any_quotes{
 sub _get_isa_values{
     my ($self,$varref) = @_;
     my @parents;
-    for my $variable(@$varref){
+
+    for my $variable ( @{ $varref || [] } ) {
         my @children = $variable->children();
-        #print Dumper($variable);
         
-        if(grep{$_->content() eq '@ISA'}@children){
-            if($variable->find_any('PPI::Statement::Expression')){
-                push(@parents,$self->_parse_expression($variable));
+        if( grep{$_->content eq '@ISA'}@children ) {
+            if( $variable->find_any('PPI::Token::QuoteLike::Words') ) {
+                push @parents, $self->_parse_quotelike($variable);
             }
-            elsif($variable->find_any('PPI::Token::QuoteLike::Words')){
-                push(@parents,$self->_parse_quotelike($variable));
+            elsif( $variable->find_any('PPI::Statement::Expression') ) {
+                push @parents, $self->_parse_expression($variable);
             }
         }
     }
+
     return @parents;
-}# _get_values
-
-sub _parse_list {
-    my ($self, $elem) = @_;
-
-    return $self->_parse_expression( $elem, 'PPI::Structure::List' );
 }
 
-sub _parse_expression{
-    my ($self, $variable, $token_class) = @_;
+sub _parse_expression {
+    my ($self, $variable) = @_;
 
-    $token_class ||= 'PPI::Statement::Expression';
-
-    my $ref = $variable->find( $token_class );
+    my $ref = $variable->find( 'PPI::Statement::Expression' );
     my @parents;
 
-    for my $element($ref->[0]->children()){
-        if($element->class =~ /^PPI::Token::Quote::/){
-            push( @parents,$element->string );
+    for my $expression ( @{$ref} ) {
+        for my $element( $expression->children ){
+            if( $element->class =~ /^PPI::Token::Quote::/ ) {
+                push @parents, $element->string;
+            }
         }
     }
 
     return @parents;
-}# _parse_expression
+}
 
 sub _parse_quotes{
     my ($self,$variable,$type) = @_;
@@ -163,8 +166,10 @@ sub _parse_quotes{
     my @parents;
     
     for my $element( $variable->children ){
-        my ($type) = ref($element) =~ /PPI::Token::Quote::([^:]+)$/;
+        my ($type) = $element->class =~ /PPI::Token::Quote::([^:]+)$/;
+
         next unless $type;
+
         my $value  = $element->string;
         push @parents, $value;
     }
@@ -174,28 +179,33 @@ sub _parse_quotes{
 
 sub _parse_quotelike{
     my ($self,$variable) = @_;
+
     my $words         = ($variable->find('PPI::Token::QuoteLike::Words'))[0]->[0];
     my $operator      = $words->{operator};
     my $section_type  = $words->{sections}->[0]->{type};
-    my ($left,$right) = split(//,$section_type);
-    $right            = $left unless defined $right;
-    (my $value        = $words->content()) =~ s~$operator\Q$left\E(.*)\Q$right\E~$1~;
-    my @parents       = split(/\s+/,$value);
+    my ($left,$right) = split //, $section_type;
+    (my $value        = $words->content) =~ s~$operator\Q$left\E(.*)\Q$right\E~$1~;
+    my @parents       = split /\s+/, $value;
+
     return @parents;
-}# _parse_quotelike
+}
 
 
 1;
 
+__END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Class::Superclasses - Find all (direct) superclasses of a class
+Class::Superclasses
 
-=head2 DESCRIPTION
+=head1 VERSION
 
-C<Class::Superclasses> uses L<PPI> to get the superclasses of a class;
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -208,6 +218,14 @@ C<Class::Superclasses> uses L<PPI> to get the superclasses of a class;
   
   print $_,"\n" for(@superclasses);
 
+=head1 NAME
+
+Class::Superclasses - Find all (direct) superclasses of a class
+
+=head2 DESCRIPTION
+
+C<Class::Superclasses> uses L<PPI> to get the superclasses of a class;
+
 =head1 METHODS
 
 =head2 new
@@ -232,4 +250,16 @@ tells C<Class::Superclasses> which Perl class should be analyzed.
 
   $parser->document($filename);
 
+=head1 AUTHOR
+
+Renee Baecker <module@renee-baecker.de>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2015 by Renee Baecker.
+
+This is free software, licensed under:
+
+  The Artistic License 2.0 (GPL Compatible)
+
 =cut
